@@ -6,6 +6,7 @@ given here.
 """
 import logging
 import os
+import urllib
 import urlparse
 from os.path import join, isfile, isdir, dirname, exists
 
@@ -71,7 +72,7 @@ def virtual_paths(request):
     starting with '%' then only the 'first' is used.
 
     """
-    if '/%' in request.fs[len(request.root):]:  # disallow direct access
+    if os.sep + '%' in request.fs[len(request.root):]:  # disallow direct access
         raise Response(404)
     if not exists(request.fs):
         matched = request.root
@@ -81,7 +82,6 @@ def virtual_paths(request):
         for i in range(1, nparts):
             part = parts[i]
             next = join(matched, part)
-            print next
             if exists(next):    # this URL part names an actual directory
                 matched = next
             else:               # this part is missing; do we have a %subdir?
@@ -96,7 +96,7 @@ def virtual_paths(request):
                             # ============================
                             # We want to allow file matches for the last URL
                             # path part, and in that case we strip the file
-                            # extension.  For other matches we need them to be
+                            # extension. For other matches we need them to be
                             # directories.
 
                             fs = join(matched, name)
@@ -104,8 +104,18 @@ def virtual_paths(request):
                             v = part
                             if i == (nparts - 1):
                                 if isfile(fs):
-                                    k = k.rsplit('.', 1)[0]
-                                    v = part.rsplit('.', 1)[0]
+                                    # Take care with extensions.
+                                    x = k.rsplit('.', 1)
+                                    y = part.rsplit('.', 1)
+                                    nx = len(x) # 1|2
+                                    if nx != len(y):
+                                        continue
+                                    if nx == 2:
+                                        # If there's an extension, match it.
+                                        k, ext1 = x
+                                        v, ext2 = y
+                                        if ext1 != ext2:
+                                            continue
                             elif not isdir(fs):
                                 continue 
 
@@ -134,11 +144,12 @@ def _typecast(key, value):
             value = int(value)
         except ValueError:
             raise Response(404)
-    else:                       # otherwise it's ASCII
+    else:                       # otherwise it's URL-quoted ASCII
         try:
             value = value.decode('ASCII')
         except UnicodeDecodeError:
             raise Response(400)
+        value = urllib.unquote(value)
     return key, value
 
 def trailing_slash(request):
